/*** 
 * \file gre/3dbound.h
 * \brief Bound structure
 *
 * \if NODOC
 * $Id: 3dbound.h_v 1.4 2006/09/15 15:29:15 mju Exp $
 *
 * $Log: 3dbound.h_v $
 * Revision 1.4  2006/09/15 15:29:15  mju
 * Serializer changes.
 *
 * Revision 1.3  2005/10/05 16:31:07  mju
 * Include gre base hdr.
 *
 * Revision 1.2  2004/12/01 20:11:57  vdronov
 * ALIGN
 *
 * Revision 1.1  2004/09/20 17:36:41  vdronov
 * Initial revision
 *
 * \endif
**/ 

#ifndef	INC_GRE_3DBOUND_H
#define	INC_GRE_3DBOUND_H

#ifndef INC_GRE_BASE_H
	#include <gre/base.h>
#endif

#ifndef INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

#ifndef	INC_MI32_SERIALIZERBASE_H
#include <mi32/serializerbase.h>
#endif

namespace GRE {

// BOUNDPARM
class BOUNDPARM {
	public:
		enum TYPE {
			TYPE_Up,
			TYPE_Down
			};

		double ALIGN8(m_Elevation);
		COLOR m_Color;					

		static const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer);

		//! Default constructor.
		BOUNDPARM (
			BOUNDPARM::TYPE type = BOUNDPARM::TYPE_Down
			) :
			m_Type(type),
			m_IsEnabled(false),				
			m_Color(0xF000, 0x8000, 0x0000),					
			m_Elevation(0.0)
			{ }

		//! Copy constructor.
		BOUNDPARM (
			const BOUNDPARM& rhs
			) :
			m_Type(rhs.m_Type),
			m_IsEnabled(rhs.m_IsEnabled),				
			m_Color(rhs.m_Color),					
			m_Elevation(rhs.m_Elevation)
			{ }

		//! Destructor.
		~BOUNDPARM() { }

		void SetEnabled (
			const bool enabled
			) { m_IsEnabled = enabled; }

		bool IsEnabled (
			) const { return m_IsEnabled; }

		TYPE GetType (
			) const { return m_Type; }

	private:
		const TYPE m_Type;
		bool m_IsEnabled;				
	};	// End of BOUNDPARM class.

}		// End of GRE namespace

#endif



