/**
 * \file gre/dlglayermgr.h
 * \brief GRE layer manager dialog.
 *
 * \if NODOC
 * $Id: dlglayermgr.h_v 1.4 2012/07/19 15:03:07 mju Exp $
 *
 * $Log: dlglayermgr.h_v $
 * Revision 1.4  2012/07/19 15:03:07  mju
 * Add getAttached.
 *
 * Revision 1.3  2011/11/28 17:07:58  mju
 * Fix dox dependency.
 *
 * Revision 1.2  2006/08/10 16:06:04  mju
 * Add placementKey parm to openSimple.
 *
 * Revision 1.1  2005/11/02 18:29:18  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_DLGLAYERMGR_H
#define	INC_GRE_DLGLAYERMGR_H

#ifndef INC_GRE_BASE_H
#include <gre/base.h>
#endif

#ifndef INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

namespace GRE {

//=====================================================================================================================

//! GRE Layer Manager dialog.
//! Provides ability to add/remove/change layers in a specified group, as well
//! as access group settings and other common operations.
//! Intended for use as modeless dialog only.
class DLG_LAYERMGR : public MGUI::DLGSHELL {
	public:

		//! Get layer manager attached to group (if any).
		static DLG_LAYERMGR* GetAttached (
			GRE::GROUP *group				//!< Group
			);

		//! Construct and open for specified group.
		//! Provides convenient way to open layer manager without need to keep track of it.
		//! If layer manager has been previously opened it will simply show it.
		static DLG_LAYERMGR* OpenSimple (
			MDLGPARENT dlgparent,		//!< Parent for dialog
			GRE::GROUP *group,			//!< Group
			const MISTRING& apptitle,	//!< Application title, will be prefixed to dialog title
			const char *PlacementKey = 0	//!< Key for retaining dialog size/placement, 0 if unused
			);

		//! Construct for specified group.
		DLG_LAYERMGR (
			GRE::GROUP *group,			//!< Group
			const MISTRING& apptitle	//!< Application title, will be prefixed to dialog title
			);

		virtual ~DLG_LAYERMGR ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		GRE::GROUP *m_group;

		// SHELL/DLGSHELL overrides.
		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnActivate (bool activated);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_DLGLAYERMGR_H
