/**
 * \file gre/legacy.h
 * \brief Legacy GRE function prototypes
 *
 * \if NODOC
 * $Id: legacy.h_v 1.32 2013/02/15 20:45:39 mju Exp $
 *
 * $Log: legacy.h_v $
 * Revision 1.32  2013/02/15 20:45:39  mju
 * Fix dox hdr.
 *
 * Revision 1.31  2013/01/22 14:33:41  mju
 * Remove unused fn.
 *
 * Revision 1.24  2008/05/27 16:09:02  mju
 * Add reason to mdispxd3dControlUpdate.
 *
 * Revision 1.20  2007/09/28 17:38:03  mju
 * Use view_legacy.
 *
 * Revision 1.15  2006/10/06 16:27:37  mju
 * Move legacy datatip form stuff here.
 *
 * Revision 1.11  2006/06/23 16:31:33  mju
 * Declare mxdAnyCallbackStruct.
 *
 * Revision 1.5  2005/11/18 17:39:32  mju
 * Use viewable as object type for select functions.
 *
 * Revision 1.1  2005/10/14 17:30:45  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LEGACY_H
#define	INC_GRE_LEGACY_H

#ifndef	INC_GRE_BASE_H
#include <gre/base.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef	INC_MI32_XDEFNS_H
#include <mi32/xdefns.h>
#endif

/*----------------------------------------------------------------------------*/

#ifndef GENERATING_DOXYGEN_OUTPUT
struct MxdAnyCallbackStruct;
struct RGNELEMLIST;
class REGION2D;
class TRANS2D_MAPGEN;
namespace MGPS {
	class SOURCE;
	}
typedef void *MXTHANDLE;
class _MDISPTOOL;
typedef _MDISPTOOL *MDISPTOOL;
#endif // GENERATING_DOXYGEN_OUTPUT


// Flags

#define	MDISP_View3dNoViewPointIcon	0x0001	//!< Don't include "viewpoint" icon on toolbar in 3D view
#define	MDISP_View3dNoStereoIcon		0x0002	//!< Don't include "stereo" icon on toolbar in 3D view

#define	MDISP_LocatorDftOpen			0x00000001	//!< Default to having locator open
#define	MDISP_LocatorViewParent		0x00000002	//!< Use reference view as parent to locator window

#define	MDISPTOOLGENERIC_NoSetTrans		0x0001	//!< Don't set tool-view transformation
#define	MDISPTOOLGENERIC_SetTransToGroup	0x0002	//!< Set transformation to be from tool<-->group


#ifndef GENERATING_DOXYGEN_OUTPUT
// Definitions for _mdispxtViewPos3D tool
struct MDISPXTVIEWPOS3DVALUES {
	GRE::VIEW_LEGACY *ToolView;				//!<  View tool is attached to 
	GRE::VIEW_LEGACY *ActiveView;			//!<  View currently having viewpoint adjusted 
	};
#endif // GENERATING_DOXYGEN_OUTPUT


//===================================================================================================================
//	Function prototypes

#ifndef GENERATING_DOXYGEN_OUTPUT
// Internal functions

extern "C" {

int				_mdispSurfaceTransCreate (void**);
void				_mdispSurfaceTransDestroy (void*);
int				_mdispSurfaceTransGetZ (void*, DPOINT3D*, INT32);
void				_mdispSurfaceTransSetInputToMap (void*, const TRANS2D_MAPGEN&);
int				_mdispSurfaceTransSetSurface (void*, GRE::LAYER_SURFACE*, double);

#ifdef X_NATIVE
MXTHANDLE		_mdispxtViewPos3D (UINT32, void*);
#endif

}

#endif // GENERATING_DOXYGEN_OUTPUT

//!\addtogroup mdisp Mdisp Functions
//!@{


// C++ linkage

int				MdispToolAdd (GRE::VIEW_LEGACY *view, MDISPTOOL *tool, const char *toolname, const char *iconname, const char *icongroup, const char *helpcode, ICID icidmenu, ICID icidicon, int(*Init)(MDISPTOOL), int(*Stop)(MDISPTOOL), int(*Activate)(MDISPTOOL), int(*Deactivate)(MDISPTOOL), int(*Suspend)(MDISPTOOL), int(*Resume)(MDISPTOOL));
inline int		MdispToolAdd (GRE::VIEW_LEGACY *view, MDISPTOOL *tool, const char *toolname, const char *iconname, const char *icongroup, const char *helpcode, int(*Init)(MDISPTOOL), int(*Stop)(MDISPTOOL), int(*Activate)(MDISPTOOL), int(*Deactivate)(MDISPTOOL), int(*Suspend)(MDISPTOOL), int(*Resume)(MDISPTOOL)) {
						return (MdispToolAdd(view, tool, toolname, iconname, icongroup, helpcode, 0, 0, Init, Stop, Activate, Deactivate, Suspend, Resume));
						}
#ifdef X_NATIVE
int				MdispToolGenericAdd (GRE::VIEW_LEGACY *view, MDISPTOOL *tool, const char *toolname, const char *iconname, const char *icongroup, const char *helpcode, ICID icidmenu, ICID icidicon, MXTHANDLE(CreateMxTool)(UINT32,void*), UINT32 flags);
inline int		MdispToolGenericAdd (GRE::VIEW_LEGACY *view, MDISPTOOL *tool, const char *toolname, const char *iconname, const char *icongroup, const char *helpcode, MXTHANDLE(CreateMxTool)(UINT32,void*), UINT32 flags) {
						return (MdispToolGenericAdd(view, tool, toolname, iconname, icongroup, helpcode, 0, 0, CreateMxTool, flags));
						}
#endif


extern "C" {	//! C Linkage

#ifdef X_NATIVE
int				Mdisp3dGroupViewPtOpen (Widget, GRE::GROUP*);
#endif

int				MdispExamineRawRaster (GRE::LAYER*);

#ifdef X_NATIVE
const MENUITEM *	MdispGPSGetMenuItem (GRE::VIEW_LEGACY*);
#endif
int				MdispGPSSetSymbolVisible (MGPS::SOURCE*, GRE::VIEW_LEGACY*, bool visible = true);
int MdispGroupControlDialogOpen (MDLGPARENT, GRE::GROUP*, UINT32 flags=0);
void				MdispLocatorCreate (GRE::VIEW_LEGACY*, UINT32);
void				MdispLocatorRaise (GRE::VIEW_LEGACY*);
void				MdispLocatorUpdate (GRE::VIEW_LEGACY*);

#ifdef X_NATIVE
int				MdispRastCorrOpen (Widget, GRE::LAYER*);
#endif

bool				MdispRastHistHasOpenViews (void);
int				MdispRastHistOpen (MDLGPARENT, GRE::LAYER*);
int				MdispRastHistSetRegion (const REGION2D& region);

#ifdef X_NATIVE
int				MdispRegionGenerateFromLayer (Widget, GRE::LAYER*, UINT32, RGNELEMLIST&);
int				MdispRegionGenerateFromTool (Widget, GRE::VIEW_LEGACY*, UINT32, ELEMTYPE, void*, INT32, const TRANS2D_MAPGEN&, const GRE::LAYERDC&, UINT32);
#endif

#ifdef X_NATIVE
BUTTONITEM *	MdispSelectGetActionButtons (GRE::VIEWABLE*);
BUTTONITEM *	MdispSelectGetModeButtons (GRE::VIEWABLE*);
#endif

#ifdef X_NATIVE
int				MdispToolAddExamRast (GRE::VIEW_LEGACY*, MDISPTOOL*);
int				MdispToolAddSelectPoint (GRE::VIEW_LEGACY*, MDISPTOOL*);
int				MdispToolAddSlideView (GRE::VIEW_LEGACY*, MDISPTOOL*);
int				MdispToolAddToolBox (GRE::VIEW_LEGACY*, MDISPTOOL*, UINT32);
int				MdispToolAddTrace (GRE::VIEW_LEGACY*, MDISPTOOL*);
int				MdispToolAddViewPos3D (GRE::VIEW_LEGACY*, MDISPTOOL*);
int				MdispToolEditGlobalOptions (Widget);
MXTHANDLE		MdispToolGenericGetMxtHandle (MDISPTOOL);
#endif

void				MdispView3dAddIcons (GRE::VIEW_LEGACY*, UINT32);
void				MdispView3dAddSimpleViewPointControls (GRE::VIEW_LEGACY*, UINT32);
void				MdispView3dControlsOpen (GRE::VIEW_LEGACY*);

#ifdef X_NATIVE
int				Mdispxd3dControlAddCallback (MXDHANDLE, void(*)(MxdAnyCallbackStruct*,void*), void*, UINT32);
int				Mdispxd3dControlDestroy (MXDHANDLE);
int				Mdispxd3dControlClose (MXDHANDLE);
int				Mdispxd3dControlCreate (Widget, MXDHANDLE*, char*, int, GRE::VIEWPOINT3D*, UINT32);
int				Mdispxd3dControlIsOpen (MXDHANDLE);
int				Mdispxd3dControlOpen (MXDHANDLE);
int				Mdispxd3dControlRemoveCallback (MXDHANDLE, void(*)(MxdAnyCallbackStruct*,void*), void*, UINT32);
int				Mdispxd3dControlSetTitle (MXDHANDLE, const MIUNICODE*);
int				Mdispxd3dControlUpdate (MXDHANDLE, UINT32 cbreason = 0);

#endif
}	//! C linkage

//!@}

#if defined(X_NATIVE) && !defined(NO_DEPRECATED)
DEPRECATED Widget MdispElementDataTipFormCreate (Widget, const RVC::OBJITEM&, ELEMTYPE, GRE::DATATIPPARMS*, UINT32, Arg*, int);
DEPRECATED void MdispElementDataTipFormUpdate (Widget, const RVC::OBJITEM&);
#endif



#endif	// INC_GRE_LEGACY_H
