/**
 * \file mgui/toolregpolygon.h
 * \brief MGUI::TOOL_REGPOLYGON class definition
 *
 * \if NODOC
 * $Id: toolregpolygon.h_v 1.8 2011/12/09 20:23:39 mju Exp $
 *
 * $Log: toolregpolygon.h_v $
 * Revision 1.8  2011/12/09 20:23:39  mju
 * No context in v_draw.
 *
 * Revision 1.7  2008/02/15 23:37:34  scowan
 * Added manual entry stubs.
 *
 * Revision 1.6  2008/02/01 15:01:41  mju
 * Change button handlers to return bool.
 *
 * Revision 1.5  2008/01/04 18:59:47  mju
 * Generalize pointer button methods.
 *
 * Revision 1.4  2007/10/05 21:12:27  mju
 * Rename inclusion guards.
 *
 * Revision 1.3  2003/10/03 19:58:19  linux32build!build
 * Doxygen
 *
 * Revision 1.2  2003/09/15 13:49:32  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.1  2003/01/10 23:08:58  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_TOOLREGPOLYGON_H
#define	INC_MGUI_TOOLREGPOLYGON_H

#ifndef	INC_MGUI_TOOLBASE_H
#include <mgui/toolbase.h>
#endif

namespace MGUI {


//! "Regular Polygon" tool
class TOOL_REGPOLYGON : public TOOL_BASE {
	public:

		//! Constructor.
		explicit TOOL_REGPOLYGON (
			TOOLCANVAS *canvas
			);

		//! Destructor.
		virtual ~TOOL_REGPOLYGON ();

		//! Get current rotation angle.
		double GetAngle (
			) const { return (m_angle); }

		//! Get center position.
		const DPOINT2D& GetCenter (
			) const { return (m_center); }

		//! Get number of segments.
		int GetNumSegments (
			) const { return (m_NumSegments); }

		//! Get radius.
		double GetRadius (
			) const { return (m_radius); }

		//! Set rotation angle.
		void SetAngle (
			double newangle
			);

		//! Set whether angle indicator is visible.
		void SetAngleVisible (
			bool anglevisible = true
			);

		//! Set center of polygon.
		void SetCenter (
			const DPOINT2D& center					//!< New center in tool coordinates
			);

		//! Set marker type and size.
		//! The default marker is MARKER_Plus with a size of 9 pixels.
		void SetMarker (
			MARKER marker,								//!< New marker type
			UINT16 size = 0							//!< New marker size, 0 to retain current size
			);

		//! Set number of polygon segments.
		void SetNumSegments (
			int NumSegments							//!< Number of segments >= 3
			);

		//! Set polygon radius (distance from center to vertex).
		void SetRadius (
			double radius								//!< Radius is tool units
			);

		//! Set allowed radius range
		void SetRadiusRange (
			double minradius,							//!< Minimum radius
			double maxradius							//!< Maximum radius, <= 0 for no maximum
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Enumeration constants
		enum ZONE {
			ZONE_NoFunction = 0,
			ZONE_Outside,
			ZONE_Inside,
			ZONE_Perimeter,
			ZONE_SetCenter,
			ZONE_Rotate,
			};
		//! Tool settings
		MARKER m_marker;
		UINT16 m_markersize;
		double m_minradius;						//!< Minimum radius
		double m_maxradius;						//!< Maximum radius, 0 if no maximum
		bool m_anglevisible;						//!< Visibility of rotation angle indicator
		//! Tool position values
		DPOINT2D m_center;						//!< Center
		double m_radius;							//!< Distance from center to each vertex
		double m_angle;
		int m_NumSegments;
		//! Values used during interaction
		ZONE m_CursorZone;						//!< Current cursor zone
		DPOINT2D m_prevpt;						//!< Previous point during button-down

		//! Manual Point Entry form forward declaration
		class MANUAL_REGPOLYGON;

		//! Methods
		void AdjustTool (const DPOINT2D&, bool firsttime=false);
		void ComputeCursorZone (const DPOINT2D&);
		void StartTool (const DPOINT2D&);
		void UpdateCursor (const DPOINT2D&) const;

		//! TOOL_BASE overrides.
		virtual MANUAL_BASE* v_CreateManualBase ();
		virtual void v_DoDraw ();
		virtual void v_DoMove (double dx, double dy, ADJUSTFLAGS flags);
		virtual int v_GenerateRegion (REGION2D& region) const;
		virtual bool v_OnKey (MGUI::POINT point, KEYCODE keycode, KEYSTATE keystate);
		virtual bool v_OnPointerButtonDown (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnPointerMove (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


}	// End namespace MGUI


#endif	// INC_MGUI_TOOLREGPOLYGON_H
