/**
 * \file elemtype.h <mi32/elemtype.h>
 * \brief Element type definitions
 *
 * \if NODOC
 * $Id: elemtype.h_v 1.11 2010/06/16 17:58:26 mju Exp $
 *
 * $Log: elemtype.h_v $
 * Revision 1.11  2010/06/16 17:58:26  mju
 * Add elemtype_mask.
 *
 * Revision 1.10  2010/02/17 22:18:12  mju
 * Add elemtypetextid_Abbreviation.
 *
 * Revision 1.2  1999/07/07  22:51:01  dwilliss
 * Added ELEMTYPE_DBCursor
 *
 * Revision 1.1  1999/04/28 14:22:14  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_ELEMTYPE_H
#define	INC_MI32_ELEMTYPE_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef MISYSTEMDLL
	#define LIBEXPORT MI_DLLEXPORT
#else
	#define LIBEXPORT MI_DLLIMPORT
#endif

enum ELEMTYPE {
	ELEMTYPE_Empty =				0,		//!< No element type
	ELEMTYPE_Node =				1,		//!< Vector or TIN node
	ELEMTYPE_Point =				2,		//!< Vector point
	ELEMTYPE_Line =				3,		//!< Vector line
	ELEMTYPE_Polygon =			4,		//!< Vector polygon
	ELEMTYPE_Label =				5,		//!< Vector label
	ELEMTYPE_CAD =					6,		//!< CAD element
	ELEMTYPE_RasterCellValue =	7,		//!< Raster image cell value
	ELEMTYPE_Record =				8,		//!< Absolute record number
	ELEMTYPE_All =					9,		//!< All element types
	ELEMTYPE_Edge =				10,	//!< TIN edge
	ELEMTYPE_Triangle =			11,	//!< TIN triangle
	ELEMTYPE_Hull =				12,	//!< TIN hull
	ELEMTYPE_Bookmark =			13,	//!< Database Bookmark record number
	ELEMTYPE_DBCursor =			14,	//!< Database cursor
	ELEMTYPE_NumTypes =			15,	//!< Number of element types
	ELEMTYPE_MASK =				0xFF	//!< Used to mask from composite value
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
#define ELEMTYPE_Poly			ELEMTYPE_Polygon
#define ELEMTYPE_RastCell		ELEMTYPE_RasterCellValue

DEFINE_ENUM_OP_INCREMENT(ELEMTYPE)
#endif

enum ELEMTYPETEXTID {
	ELEMTYPETEXTID_SingularLowercase,
	ELEMTYPETEXTID_SingularCapitalize,
	ELEMTYPETEXTID_PluralLowercase,
	ELEMTYPETEXTID_PluralCapitalize,
	ELEMTYPETEXTID_Abbreviation
	};

//! Get TEXTID for specified element type and name type.
LIBEXPORT TEXTID GetElemTypeTextID (
	ELEMTYPE ElemType,					//!< Element type
	ELEMTYPETEXTID NameType				//!< Type of name to return TEXTID for
	);

#undef	LIBEXPORT

#endif
