/**
 * \file mi32/filetypeset.h
 * \brief FILETYPESET class definition
 *
 * \if NODOC
 * $Id: filetypeset.h_v 1.3 2009/11/05 22:07:19 mju Exp $
 *
 * $Log: filetypeset.h_v $
 * Revision 1.3  2009/11/05 22:07:19  mju
 * Add isEmpty.
 *
 * Revision 1.2  2009/11/04 22:07:35  mju
 * *** empty log message ***
 *
 * Revision 1.1  2009/11/04 18:56:38  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_FILETYPESET_H
#define	INC_MI32_FILETYPESET_H

#ifndef	INC_MI32_FILETYPE_H
#include <mi32/filetype.h>
#endif

#include <map>

#ifdef RVCSYSDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

//=====================================================================================================================

typedef MILIST<FILETYPE> FILETYPELIST;
typedef std::multimap<MISTRING,FILETYPE> EXTN_FILETYPE_MAP;
typedef std::pair<EXTN_FILETYPE_MAP::const_iterator,EXTN_FILETYPE_MAP::const_iterator> EXTN_FILETYPE_ITERATOR_PAIR;

//=====================================================================================================================

//! Set of FILETYPEs, allows access to list and map from extension to associated types.
class CLASSLIBEXPORT FILETYPESET {
	public:

		//! Get common filetype set.
		static const FILETYPESET& GetCommon ();

		//! Add FILETYPE to set.
		void Add (
			const FILETYPE& FileType,
			bool OnlyIfExtnUnique = false
			);

		//! Get FILETYPE list.
		const FILETYPELIST& GetList (
			) const { return (m_List); }

		//! Get map from extension MISTRING to FILETYPE.
		const EXTN_FILETYPE_MAP& GetMap (
			) const { return (m_Map); }

		//! Determine if set is empty.
		bool IsEmpty (
			) const { return (m_List.IsEmpty()); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FILETYPELIST m_List;
		EXTN_FILETYPE_MAP m_Map;
		#endif
	};

//=====================================================================================================================

#undef	CLASSLIBEXPORT
#endif	// INC_MI32_FILETYPESET_H
