/*****************************************************************************
 *
 * \file mi32/htmlenum.h
 * \brief HTML::ENUM and support classes
 *
 * $Log: htmlenum.h_v $
 * Revision 1.1  2007/08/29 15:42:10  dwilliss
 * Initial revision
 *
 *
 * Splitting up htmldoc.h.
 **/

#ifndef INC_MI32_HTMLENUM_H
#define INC_MI32_HTMLENUM_H

namespace HTML {

	//! A template for any enum in a CSS STYLE.
	//! An HTML::ENUM can have a value or it can be "not set".  It can also
	//! be parsed given an array of HTML::ENUM::BINDINGs
	template<typename _ET> class ENUM {
		public:

			struct BINDING {
				const char* name;
				_ET val;
				};

			//! Default ctor -- initializes to a "not set" state
			ENUM() :
				m_Value(static_cast<_ET>(0)),
				m_bSet(false)
				{ }

			//! ctor -- initialize to a specific value
			ENUM(_ET val) :
				m_Value(val),
				m_bSet(true)
				{ }
			
			//! copy ctor
			ENUM(const ENUM<_ET>& rhs) :
				m_Value(rhs.m_Value),
				m_bSet(rhs.m_bSet)
				{ }
			
			//! Convert to the actual enum
			operator _ET (
				) const { return (m_Value); }

			//! Assignment operator
			ENUM<_ET>& operator=(_ET val) {
				m_Value = val;
				m_bSet = true;
				return (*this);
				}

			//! Clear to "not set" state
			void Clear() {
				m_bSet = false;
				}

			//! Get the value
			_ET GetValue (
				) const { return (m_Value); }

			//! Determine if the value is set or not
			bool IsSet (
				) const { return (m_bSet); }

			//! Combine with another enum.
			//! If the rhs has a value, this is given that value.  
			//! If rhs is not set, the value of this is left alone.
			void Combine (
				const ENUM<_ET>& rhs
				) {
				if (rhs.m_bSet) {
					m_Value = rhs.m_Value;
					m_bSet = true;
					}
				}

			//! Parse a string containing the value of the enum.
			//! Must be given a static array of HTML::ENUM::BINDING which maps
			//! the strings to actual enum values.  The last element of the array
			//! must be have a NULL string for the name.
			bool Parse (
				const char* str,
				const BINDING* bindings,
				bool bClearIfNotFound = true
				) {
				for (int i = 0 ; bindings[i].name != 0 ; ++i) {
#ifdef WIN32
					if (!_stricmp(bindings[i].name, str)) 
#else
					if (!stricmp(bindings[i].name, str)) 
#endif
						{
						*this = bindings[i].val;
						return (true);
						}
					}
				if (bClearIfNotFound) Clear();
				return (false);
				}

#ifndef GENERATING_DOXYGEN_OUTPUT
		private:
			_ET m_Value;
			bool m_bSet;
#endif
		};


	} // end of HTML namespace


#endif

