/**
 * \file linerast.h <mi32/linerast.h>
 * \brief Definitions for LINERASTERIZER class
 *
 * \if NODOC
 * $Id: linerast.h_v 1.3 2007/01/31 17:38:08 vdronov Exp $
 *
 * $Log: linerast.h_v $
 * Revision 1.3  2007/01/31 17:38:08  vdronov
 * added LINERASTERIZER3D.
 *
 * Revision 1.2  2006/09/20 17:31:34  scowan
 * Fixed include file guard and fwd decls.
 *
 * Revision 1.1  2005/10/27 17:50:25  vdronov
 * Initial revision
 *
 * \endif
**/


#ifndef	INC_MI32_LINERAST_H
#define	INC_MI32_LINERAST_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

class POLYLINE;

class LINERASTERIZER2D {
	public:

	class SPAN {
		public:

			void Process (const INT32 colmin, const INT32 colmax, const INT32 lin) {v_Process(colmin, colmax, lin);}; 
			void ProcessSpan (const double x1, const double y1, const double x2, const double y2) {v_ProcessSpan (x1, y1, x2, y2);};

		private:

			virtual void v_Process (const INT32 colmin, const INT32 colmax, const INT32 lin) = 0; 
			virtual void v_ProcessSpan (const double x1, const double y1, const double x2, const double y2);
		};

	static void MakeScans (
		const POLYLINE& polyline,
		SPAN& span
		);

	private:
		// Not implemented
		LINERASTERIZER2D();
		~LINERASTERIZER2D();
		LINERASTERIZER2D(const LINERASTERIZER2D&);

	};

class LINERASTERIZER3D {
	public:

	class SPAN {
		public:

			void ProcessCell (const double x1, const double y1, const double z1, const double x2, const double y2, const double z2) { v_ProcessCell(x1, y1, z1, x2, y2, z2); } 
			void ProcessSpan (const double x1, const double y1, const double z1, const double x2, const double y2, const double z2) { v_ProcessSpan (x1, y1, z1, x2, y2, z2); }

		private:

			virtual void v_ProcessCell (const double x1, const double y1, const double z1, const double x2, const double y2, const double z2) = 0; 
			virtual void v_ProcessSpan (const double x1, const double y1, const double z1, const double x2, const double y2, const double z2);
		};

	static void MakeScans (
		const POLYLINE& polyline,
		SPAN& span
		);

	private:
		// Not implemented
		LINERASTERIZER3D();
		~LINERASTERIZER3D();
		LINERASTERIZER3D(const LINERASTERIZER3D&);

	};

#endif	//!< INC_MI32_LINERAST_H

