/**
 *	\file smlfuncs.h <mi32/smlfuncs.h>
 *	\brief List of all the install functions for SML.
 *
 * \if NODOC
 * $Id: smlfuncs.h_v 1.92 2012/05/10 13:17:45 mju Exp $
 *
 *	If included without defining anything, it just defines the prototypes
 *
 *	If included with
 *		SML_DEFINE_QUERY_DLL_FUNC_LIST
 *		SML_DEFINE_QUERY_FUNC_LIST
 *		SML_DEFINE_GEOFORMULA_FUNC_LIST
 *		SML_DEFINE_SML_FUNC_LIST
 *
 * $Log: smlfuncs.h_v $
 * Revision 1.92  2012/05/10 13:17:45  mju
 * Remove unused legacy prototypes.
 *
 * Revision 1.91  2011/10/12 13:57:40  mju
 * Remove TNTsim3d classes as product no longer supported.
 *
 * Revision 1.90  2008/12/17 22:57:37  bcolombe
 * Added InstallMIJOBClass
 *
 * Revision 1.89  2008/10/15 20:11:40  mju
 * Don't include sml hdr, do include stddefns.
 *
 * Revision 1.88  2008/02/15 18:26:48  bcolombe
 * Moved image pipeline classes to tntdisp only
 *
 * Revision 1.87  2007/05/08 15:52:54  dwilliss
 * SML namespace
 *
 * Revision 1.86  2007/04/05 21:30:47  dwilliss
 * changed export defines
 *
 * Revision 1.84  2007/02/14 18:53:47  drb
 * Added DBase functions so I can add enums
 *
 * Revision 1.82  2007/01/26 22:13:33  drb
 * Added Cutfill, DBase, DBTable autoclass installers
 *
 * Revision 1.81  2007/01/12 15:56:54  drb
 * Added SVG install pointer
 *
 * Revision 1.80  2007/01/10 14:42:14  dwilliss
 * replaced a prototype that shouldn't have been deleted yet.
 *
 * Revision 1.79  2007/01/09 16:13:34  dwilliss
 * Reorganized int one big list -- no more splitting up functions by useage
 *
 * Revision 1.78  2007/01/03 23:23:04  drb
 * Move GPSDB function call
 *
 * Revision 1.75  2006/11/30 15:44:25  drb
 * Set pipeline func installer to return int.
 *
 * Revision 1.74  2006/11/29 23:43:47  drb
 * Added PipelineClasses function
 *
 * Revision 1.71  2005/10/19 16:55:30  mju
 * Don't install frame functions/classes in base funclist as they depend on GRE functions.
 *   Frame classes/functions are now installed in qqmdisp.c
 *
 * Revision 1.67  2005/02/24 18:12:39  dwilliss
 * Added HTTP classes
 *
 * Revision 1.66  2004/10/11 16:32:56  vdronov
 * added Triangulator class
 *
 * Revision 1.62  2004/02/17 18:07:36  mju
 * Don't install tip functions/classes unless X or MFC as not usable.
 *
 * Revision 1.57  2002/12/09 17:25:35  dwilliss
 * Moved the MDISP function list to smlgre.c.  Only installed if calling
 * program calls SML_EnableGRE()
 *
 * Revision 1.56  2002/08/16 20:19:12  dwilliss
 * Added XML classes
 *
 * Revision 1.55  2002/05/14 17:01:21  mju
 * Don't install object conv funcs for geoformula unless X (and shouldn't need then either).
 *
 * Revision 1.50  2001/04/09 17:53:53  dwilliss
 * Added a tntdisp only function group again
 *
 * Revision 1.40  2000/05/26 19:26:22  mju
 * Add EnableImport and remove InstallImportExport.
 *
 * \endif
**/

#ifndef	INC_MI32_SMLFUNCS_H
#define	INC_MI32_SMLFUNCS_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

namespace SML {
	class CONTEXT;
	typedef int (*FUNCINSTALLER)(CONTEXT *);
	}

//! Enable import/export functions/classes in general SML contexts.
//! The programmer is responsible for setting up the MIE system to add
//! any desired formats, using the MieAddFormatsAll() or related functions.
void SML_EnableImportExport ();

//! Enable the GRE functions and classes in SML.  This should be called from
//! main() of any program which will create an SML context and expects the
//! GRE functions to be there.
void SML_EnableGRE ();

#ifndef GENERATING_DOXYGEN_OUTPUT
// Note.  Everything from here to the end of the file is stuff that
// should not be documented.  This include file should really be in
// inc/mi32p.  If we do that, the above stuff should be moved to sml.h
// and any module that installs sml functions or classes would need to
// include this file directly.  They currently include it indirectly
// through sml.h which includes it.   Failing to include this file
// the modules would still compile but their installer functions would
// end up getting C++ linkage instead of C and cause unresolved externals
// later.

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
#endif

#if defined(__cplusplus)
extern "C" {
#endif

#ifndef SML_FUNCLIST_PROTOS_DEFINED
#define SML_FUNCLIST_PROTOS_DEFINED

/* function prototypes for new sml installs */
int SML_InstallRasterClasses(SML::CONTEXT *);
int SML_InstallRasterFunctions(SML::CONTEXT *);
int SML_InstallFilterFunctions(SML::CONTEXT *);
int SML_InstallFFTClasses(SML::CONTEXT *);
int SML_InstallVectorClasses(SML::CONTEXT *);
int SML_InstallVectorFunctions(SML::CONTEXT *);
int SML_InstallCADFunctions(SML::CONTEXT *);
int SML_InstallTINFunctions(SML::CONTEXT *);
int _qq_install_matrix_functions(SML::CONTEXT *);					/* in qqmatrix.c - matrix functions */
int _qq_install_linestyle_functions(SML::CONTEXT *);				/* in qqstyle.c - line style functions */
int SML_InstallVectorToolkitFunctions(SML::CONTEXT *);		/* in smlvkit.c - vector toolkit functions */
int SML_InstallClassifyFunctions(SML::CONTEXT *);				/* in smlclass.c  - high level classify functions */
int SML_InstallColorConversionFunctions(SML::CONTEXT *);
int SML_InstallObjectConversionFunctions(SML::CONTEXT *);
int SML_InstallRegionFunctions(SML::CONTEXT *);

int _qq_install_math_functions(SML::CONTEXT *);
int _qq_install_georef_functions(SML::CONTEXT *);
int SML_InstallGeorefFunctions(SML::CONTEXT *);
int SML_InstallGeorefClasses(SML::CONTEXT *);
int SML_InstallSpatRefClasses(SML::CONTEXT *);
void SML_InstallMapProjClass(SML::CONTEXT *);
int SML_InstallObjectFunctions(SML::CONTEXT *);
RVCSYSLIBEXPORT int SML_InstallObjectClasses(SML::CONTEXT *);
int SML_InstallObjectClasses2(SML::CONTEXT *);
int SML_InstallPointSymbolFunctions(SML::CONTEXT *);

int SML_InstallConsoleClasses(SML::CONTEXT *context);
int SML_InstallConsoleFunctions(SML::CONTEXT *context);
RVCSYSLIBEXPORT int SML_InstallStringFunctions(SML::CONTEXT *context);
int SML_InstallPopupFunctions(SML::CONTEXT *context);
int SML_InstallPortClasses(SML::CONTEXT *context);
int SML_InstallPortFunctions(SML::CONTEXT *context);
int SML_InstallFileClasses(SML::CONTEXT *context);
int SML_InstallFileFunctions(SML::CONTEXT *context);
int SML_InstallIniFileClasses(SML::CONTEXT *context);
int SML_InstallIniFileFunctions(SML::CONTEXT *context);
int SML_InstallIniPortClasses(SML::CONTEXT *context);
int SML_InstallIniPortFunctions(SML::CONTEXT *context);
int SML_InstallStandardClasses(SML::CONTEXT *context);
int SML_InstallColorPaletteFunctions(SML::CONTEXT *context);
int SML_InstallPrintToBufferFunctions(SML::CONTEXT *);
int SML_InstallStyleFunctions(SML::CONTEXT *context);
int SML_InstallSystemFunctions(SML::CONTEXT *context);

int SML_InstallWidgetClasses(SML::CONTEXT *);
int SML_InstallWidgetFunctions(SML::CONTEXT *);
int SML_InstallTipClasses(SML::CONTEXT *);
int SML_InstallTipFunctions(SML::CONTEXT *);

int SML_InstallMgdFuncs(SML::CONTEXT *);
int SML_InstallMgdClasses(SML::CONTEXT *);
int SML_InstallHTTPClasses(SML::CONTEXT *);
int SML_InstallInternalSymbolFuncs(SML::CONTEXT *);
int _SML_InstallDBEDITFuncs(SML::CONTEXT *);
int _SML_InstallDBEDITClasses(SML::CONTEXT *);
int _SML_InstallMdispFuncs(SML::CONTEXT *);
int _SML_InstallMdispClasses(SML::CONTEXT *);
int _SML_InstallMdispLayerFuncs(SML::CONTEXT *);
int _SML_InstallMxdwFuncs(SML::CONTEXT *);		/* Should go away when disp2d does */

#if defined(WIN32_MFC) || !defined(WIN32_NATIVE)
int SML_InstallGPSClasses(SML::CONTEXT *context);
int SML_InstallGPSFunctions(SML::CONTEXT *context);
#endif
int SML_InstallGeoFormulaFunctionList(SML::CONTEXT *context);
int SML_InstallRunFunctions(SML::CONTEXT *);

int SML_InstallNetworkClasses(SML::CONTEXT *);
int SML_InstallNetworkFunctions(SML::CONTEXT *);

int SML_InstallWatershedClasses(SML::CONTEXT *);
int SML_InstallWatershedFunctions(SML::CONTEXT *);

int SML_InstallMovieClasses(SML::CONTEXT *);
int SML_InstallMovieFunctions(SML::CONTEXT *);

int SML_InstallFrameClasses(SML::CONTEXT *);
int SML_InstallFrameFunctions(SML::CONTEXT *);

int SML_InstallSurfaceFunctions(SML::CONTEXT *);

int SML_InstallTriangulatorFunctions(SML::CONTEXT *);
/*----------------*/

RVCSYSLIBEXPORT int SML_ResolveClassMembers(SML::CONTEXT *);
RVCSYSLIBEXPORT int SML_InstallAutoClassesCompField (SML::CONTEXT* context);

RVCSYSLIBEXPORT int SML_InstallAutoClassesSelectQuery (SML::CONTEXT* context);
RVCSYSLIBEXPORT int SML_InstallAutoClassesStyleQuery (SML::CONTEXT* context);
RVCSYSLIBEXPORT int SML_InstallAutoClassesDispLayer (SML::CONTEXT* context);
RVCSYSLIBEXPORT int SML_InstallAutoClassesGeoFormula (SML::CONTEXT* context);
RVCSYSLIBEXPORT int SML_InstallAutoClassesPinMap (SML::CONTEXT* context);
RVCSYSLIBEXPORT int SML_InstallAutoClassesSML (SML::CONTEXT* context);
int SML_InstallGUIControlClasses(SML::CONTEXT *);
int SML_InstallGUIDataClasses(SML::CONTEXT *);
int SML_InstallXMLClasses(SML::CONTEXT *);
int SML_InstallGlgClasses(SML::CONTEXT *);	// glg library incompatible with VC8 -- only exists in tnt72
int SML_InstallEXIFClass(SML::CONTEXT *);
int SML_InstallGPSDBASEClass(SML::CONTEXT *context);
int SML_InstallImagePipelineClasses(SML::CONTEXT *);
int SML_InstallMIJOBClass(SML::CONTEXT *);
int SML_InstallSVGClasses(SML::CONTEXT *);
int SML_InstallAutoClassesCutFill(SML::CONTEXT *);
int SML_InstallDBaseFunctions(SML::CONTEXT *);
int SML_InstallAutoClassesDBase(SML::CONTEXT *);
int SML_InstallAutoClassesDBTable(SML::CONTEXT *);

#ifdef GLGDLL
MI_DLLEXPORT
#else
MI_DLLIMPORT
#endif
int SML_InstallGlgClassesDll(SML::CONTEXT *);


#endif

/*-----------------------------------------------------*/
#ifdef SML_DEFINE_RVCSYS_FUNC_LIST
// This is the list of SML function installers in rvcsys.
// These functions will be installed for every query.

SML::FUNCINSTALLER SML_RVCSYSFuncList[] = {
	_qq_install_math_functions,				//!<  qqmath.c (rvcapp.dll)
	SML_InstallStringFunctions,				//!<  qqstring.c (rvcapp.dll)
	SML_InstallObjectFunctions,				//!<  qqobject.c (rvcapp.dll)
	NULL
	};

#endif


/*-----------------------------------------------------*/
#ifdef SML_DEFINE_ALL_INSTALLERS

// Register everything here..
SML::FUNCINSTALLER SML_AllInstallers[] = {
	// Classes
	SML_InstallStandardClasses,
	SML_InstallObjectClasses,
	SML_InstallObjectClasses2,
	SML_InstallFileClasses,
	SML_InstallPortClasses,
	SML_InstallIniFileClasses,
	SML_InstallRasterClasses,
	SML_InstallVectorClasses,
	SML_InstallMgdClasses,
	SML_InstallHTTPClasses,
	SML_InstallSpatRefClasses,
	SML_InstallNetworkClasses,
	SML_InstallEXIFClass,
	SML_InstallGPSDBASEClass,
	SML_InstallWidgetClasses,					//!<  smlxwidg.c
	SML_InstallAutoClassesCompField,
	SML_InstallAutoClassesSelectQuery,
	SML_InstallAutoClassesStyleQuery,
	SML_InstallAutoClassesDispLayer,
	SML_InstallAutoClassesPinMap,
	SML_InstallAutoClassesSML,
	SML_InstallAutoClassesGeoFormula,
	SML_InstallGUIControlClasses,
	SML_InstallXMLClasses,
	SML_InstallSVGClasses,						//!<  svg.c
	SML_InstallAutoClassesCutFill,				//!<  rvcopcutfill.c
	SML_InstallAutoClassesDBase,				//!<  smldbase.c
	SML_InstallAutoClassesDBTable,				//!<  smldbtable.c

	// Functions
	_qq_install_matrix_functions,				//!<  qqmatrix.c
	SML_InstallGeorefFunctions,					//!<  qqgeoref.c
	SML_InstallGeorefClasses,
	SML_InstallSpatRefClasses,					//!<  smlspatref.c
	SML_InstallColorPaletteFunctions,			//!<  qqstruct.c
	SML_InstallStyleFunctions,					//!<  qqstruct.c
	SML_InstallSystemFunctions,					//!<  qqstruct.c
	SML_InstallColorConversionFunctions,		//!<  smlconv.c
	SML_InstallPopupFunctions,					//!<  smlpopup.c
	SML_InstallFileFunctions,					//!<  smlfile.c
	SML_InstallPortFunctions,					//!<  smlport.c
	SML_InstallIniFileFunctions,				//!<  smlini.c

	SML_InstallObjectConversionFunctions,		//!<  smlconv.c
	SML_InstallRasterFunctions,					//!<  qqrast.c
	SML_InstallFilterFunctions,					//!<  qqfilt.c
	SML_InstallCADFunctions,					//!<  qqcad.c
	SML_InstallTINFunctions,					//!<  qqtin.c
	SML_InstallVectorFunctions,					//!<  qqvect.c
	SML_InstallVectorToolkitFunctions,			//!<  smlvkit.c
	SML_InstallRegionFunctions,					//!<  smlrgn.c
	SML_InstallClassifyFunctions,				//!<  smlclsfy.c
	SML_InstallConsoleFunctions,				//!<  qqprint.c
	SML_InstallMgdFuncs,						//!<  qqmgd.c
	SML_InstallInternalSymbolFuncs,				//!<  smlinit.c
	SML_InstallWidgetFunctions,					//!<  smlxwidg.c 
	SML_InstallWatershedClasses,			  	//!<  watermdl.c
	SML_InstallWatershedFunctions,		  		//!<  watermdl.c
#if defined(X_NATIVE) || defined(WIN32_MFC)
	SML_InstallTipClasses,						//!<  mguitooltip.c
	SML_InstallTipFunctions,					//!<  mguitooltip.c
#endif
	SML_InstallSurfaceFunctions,				//!<  qqsfmod.c
	SML_InstallFFTClasses,						//!<  fftfunc.c
	SML_InstallTriangulatorFunctions,			//!<  qqtriang.c
	_qq_install_linestyle_functions,			//!<  qqstyle.c
	SML_InstallPointSymbolFunctions,			//!< qqpoints.c

	SML_InstallRunFunctions,					//!<  smlxmain.c

/*-------------------------------------*/
	SML_ResolveClassMembers,	//!<  Put this one last!
	NULL
	};

#endif

#ifdef SML_DEFINE_SML_TNTDISP_ONLY_FUNCS
//! Functions in this list are only installed in TNTdisp (and SML/W)
//! Installed by tntdisp/tvmain.c 
SML::FUNCINSTALLER SML_TNTDispFuncList[] = {
	SML_InstallMovieClasses,			  		//!<  movienc.c
	SML_InstallMovieFunctions,		  			//!<  movienc.c
	SML_InstallImagePipelineClasses,
	NULL
	};


#endif

#if defined(__cplusplus)
}
#endif

#undef RVCSYSLIBEXPORT

#endif	// GENERATING_DOXYGEN_OUTPUT
#endif	// INC_MI32_SMLFUNCS_H
