/**
 * \file <mi32/spovrgn.h>
 * \brief SURFACE_REGION_PROP_CALC definitions.
 *
 * \if NODOC
 * $Id: spovrgn.h_v 1.2 2010/11/05 13:36:06 mju Exp $
 *
 * $Log: spovrgn.h_v $
 * Revision 1.2  2010/11/05 13:36:06  mju
 * Use triangulator and don't inline ctor/dtor.
 *
 * Revision 1.1  2006/06/01 20:51:24  vdronov
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_SPOVRGN_H
#define	INC_MI32_SPOVRGN_H

#ifndef	INC_MI32_TRIANGULATOR_H
#include <mi32/triangulator.h>
#endif

#ifndef	INC_RVC_RASTER_H
#include <rvc/raster.h>
#endif

#ifndef	INC_RVC_NULLMASK_H
#include <rvc/nullmask.h>
#endif


//! Calculator of surface properties over region.
class SURFACE_REGION_PROP_CALC {
	public:

		struct PROPERTIES {
			double m_ZLevel;
			double m_SurfaceArea;
		  	double m_VolumePositive;
		  	double m_VolumeNegative;
			double m_MinZ;
			double m_MaxZ;
			bool m_Full;

			PROPERTIES (
				) : 
				m_ZLevel(0.0),
				m_SurfaceArea(0.0),
		  		m_VolumePositive(0.0),
		  		m_VolumeNegative(0.0),
				m_MinZ(DBL_MAX),
				m_MaxZ(-DBL_MAX),
				m_Full(true)
				{ }

			void Clear (
				) {
				m_SurfaceArea = 0.0;
		  		m_VolumePositive = 0.0;
		  		m_VolumeNegative = 0.0;
				m_MinZ = DBL_MAX;
				m_MaxZ = -DBL_MAX;
				m_Full = true;
				}
			};

		SURFACE_REGION_PROP_CALC ();

		~SURFACE_REGION_PROP_CALC ();

		void Clear ();

		bool Initialize (
			RVC::OBJITEM& objitem
			);

		bool IsInitialized () { return m_Initialized; };

		ERRVALUE Compute (
			const REGION2D& region,
			PROPERTIES& properties
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::RASTER m_Surface;
		RVC::NULLMASK m_NullMask;
		RVC::RASTER::IMAGESCALE m_Scale;
		DPOINT2D m_NRScale;
		DRECT2D m_Extents;

		TRIANGULATOR m_Triangulator;
		DOUBLE_ARRAY<DTRIANGLE2D> m_Triangles;

		bool m_Initialized;
		PROPERTIES* m_Properties;

		ERRVALUE ProcessRegion(const REGION2D& region);	
		void ControlProcess (const DPOINT2D& p1, const DPOINT2D& p2, const DPOINT2D& p3, const int mode);

		void SplitY (const DPOINT2D& p1, const DPOINT2D& p2, const DPOINT2D& p3, const int mode);
		void SplitEQY (const DPOINT2D& p1, const DPOINT2D& p2, const DPOINT2D& p3, const int mode);
		void SplitNEQY (const DPOINT2D& p1, const DPOINT2D& p2, const DPOINT2D& p3, const int mode);
		void SplitTtoB (const DPOINT2D& top, const DPOINT2D& b1, const DPOINT2D& b2, const int mode);
		void SplitBtoT (const DPOINT2D& bottom, const DPOINT2D& t1, const DPOINT2D& t2, const int mode);

		void Process (const DPOINT2D& p1, const DPOINT2D& p2, const DPOINT2D& p3);
		bool CalculatePlane (const INT32 column, const INT32 line, const int status, DPOINT3D& plane);
		bool GetElevation(const INT32 column, const INT32 line, double& z);
			
		void ComputeVolume (const DPOINT3D& p1, const DPOINT3D& p2, const DPOINT3D& p3, double& vp, double& vn);
		double ComputeTrianglePyramidVolume (const DPOINT3D& p1, const DPOINT3D& p2, const DPOINT3D& p3);

		#endif	// GENERATING_DOXYGEN_OUTPUT
	};

#endif	// INC_MI32_SPOVRGN_H
