/**
 * \file rvc/dbpoly.h
 * \brief interface for the RVCDBASE_POLYGON class.
 *
 * \if NODOC
 * $Id: dbpoly.h_v 1.16 2010/05/12 21:07:53 mju Exp $
 *
 * $Log: dbpoly.h_v $
 * Revision 1.16  2010/05/12 21:07:53  mju
 * Simplify and specify parent objtype when construct base.
 *
 * Revision 1.15  2008/08/27 22:02:25  scowan
 * Added parm to combine method to take an optional trans2d-mapgen.
 *
 * Revision 1.12  2004/10/04 16:56:42  scowan
 * Added parameter to combine polygon attrib method.
 *
 * Revision 1.10  2004/02/09 20:14:34  scowan
 * Added parameter to combine attrib to deal with src polys not transfered.
 *
 * Revision 1.9  2003/10/07 19:44:22  scowan
 * Added method to combine attributes from multiple source polygons to a destination polygon vector object.
 *
 * Revision 1.1  2001/05/24 15:49:07  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef INC_RVC_DBPOLY_H
#define INC_RVC_DBPOLY_H

#ifndef INC_RVC_DBELEM_H
	#include <rvc/dbelem.h>
#endif

#ifndef INC_MI32_BITSETD_H
	#include <mi32/bitsetd.h>
#endif

#ifndef INC_MI32_REGION2D_H
	#include <mi32/region2d.h>
#endif

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
class VECTOR;		// Forward declarations
#endif // GENERATING_DOXYGEN_OUTPUT

//! Manage RVC::OBJTYPE_Database_VectorPolygon objects.
class DBASE_POLYGON : public RVC::DBASE_ELEMENT {
	public:

		//! Default constructor
		DBASE_POLYGON (
			): DBASE_ELEMENT(OBJTYPE_Vector,ELEMTYPE_Polygon) { }

		//! Combine polygon attributes from multiple geometric sources into the destination vector polygon database object
		//! This method will remove the standard attributes tables, caller will need to rebuild
		ERRVALUE CombinePolygonAttributesFrom (
			const RVC::OBJITEMLIST& GeoObjItemList,
			DBASE::COPYFLAG CopyFlags,
			DBASE::JOINFLAGS JoinFlags,
			const MILIST<BITSET_DEQUE> *pValidPolyList = 0,		//!< If specified, the number of entries must match 'GeoObjItemList' entries
																				//!< Determines which polygons from the source are to be used
			const MILIST<REGION2D> *pValidRegionList = 0,			//!< Determines which polygons from the destination are to be used
			const TRANS2D_MAPGEN *pDestToSrcAlt = 0					//!< Use if the translation between the source and dest is not from RVC::GEOSPATIAL::ComputeObjectTranslation()
			);
			
	};

}		// End of RVC namespace

#endif 	// INC_RVC_DBPOLY_H
