/**
 * \file rvc/uiimagepyramid.h
 * \brief RVC::UI::FORM_IMAGE_PYRAMID class.
 *
 * \if NODOC
 * $Id: uiimagepyramid.h_v 1.5 2008/09/02 21:43:25 mju Exp $
 *
 * $Log: uiimagepyramid.h_v $
 * Revision 1.5  2008/09/02 21:43:25  mju
 * Fix doxygen warning.
 *
 * Revision 1.4  2007/07/18 16:10:18  mju
 * Don't pass flags by ref in setvalues.
 *
 * Revision 1.3  2006/07/12 14:17:47  mju
 * Add getLabel.
 *
 * Revision 1.2  2006/06/28 15:35:41  mju
 * Rename as form_image_pyramid.
 *
 * Revision 1.1  2006/06/23 14:37:26  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIIMAGEPYRAMID_H
#define	INC_RVC_UIIMAGEPYRAMID_H

#ifndef	INC_RVC_IMAGE_H
#include <rvc/image.h>
#endif

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif


namespace RVC {
namespace UI {

//===================================================================================================================
//! Form for setting IMAGE::PYRAMID values.
class FORM_IMAGE_PYRAMID : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		FORM_IMAGE_PYRAMID ();

		//! Destructor.
		virtual ~FORM_IMAGE_PYRAMID ();

		//! Create form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane				//!< Parent pane
			);

		//! Get label control.
		//! Should only be used for alignment purposes.
		MGUI::CTRL_LABEL& GetLabel (
			) { return (m_CbxPyramid.GetLabel()); }

		//! Get current values.
		void GetValues (
			bool& DoPyramid,										//!< Return, true if should make pyramid tiers, false if not
			RVC::IMAGE::PYRAMID& flags							//!< Return, pyramid flags
			);

		//! Set delegate to call on any change.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChange = delegate; }

		//! Set values.
		void SetValues (
			bool DoPyramid,										//! True if should make pyramid tiers, false if not
			RVC::IMAGE::PYRAMID flags,							//! Pyramid flags
			bool notify												//! Send notification via delegate
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_IMAGE_PYRAMID THISCLASS;
		MGUI::FORM_COMBOBOX m_CbxPyramid;
		DELEGATE_VOID_NOPARMS m_DelegateOnChange;
		void NotifyChange ();
		FORM_IMAGE_PYRAMID (const FORM_IMAGE_PYRAMID&);
		FORM_IMAGE_PYRAMID& operator= (const FORM_IMAGE_PYRAMID&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================


}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIIMAGEPYRAMID_H
