/**
 * \file spatmod/imagesourcepng.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_PNG definition.
 *
 * \if NODOC
 * $Id: imagesourcepng.h_v 1.5 2011/10/11 14:44:07 mju Exp $
 *
 * $Log: imagesourcepng.h_v $
 * Revision 1.5  2011/10/11 14:44:07  mju
 * Remove unneeded hdrs.
 *
 * Revision 1.4  2009/02/19 15:07:51  mju
 * Add setTransparentInvalid.
 *
 * Revision 1.1  2007/01/24 22:45:38  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCEPNG_H
#define	INC_SPATMOD_IMAGESOURCEPNG_H

#ifndef	INC_SPATMOD_IMAGESOURCEFILE_H
#include <spatmod/imagesourcefile.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Pipeline source for reading PNG images from memory or file.
class SOURCE_PNG : public SOURCE_FILE {
	public:

		//! Construct with FILEPATH.
		explicit SOURCE_PNG (
			const FILEPATH& FilePath
			);

		//! Construct with memory buffer (SIMPLE_ARRAY).
		//! Will take ownership of buffer.
		explicit SOURCE_PNG (
			SIMPLE_ARRAY<UINT8>& MemBuffer	//!< Memory buffer, will be returned empty
			);

		//! Construct with memory buffer (pointer/size).
		//! Will take ownership of buffer, pointer will be returned as NULL
		SOURCE_PNG (
			UINT8*& MemBuffer,					//!< Memory buffer, will be returned as NULL
			UINT32 BufferSize						//!< Buffer size
			);

		//! Destructor.
		virtual ~SOURCE_PNG ();

		//! Set to treat fully transparent pixels as "invalid".
		void SetTransparentInvalid (
			bool TransparentInvalid = true
			) { m_TransparentInvalid = TransparentInvalid; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PNG_READER;
		PNG_READER *m_pReader;
		bool m_DoingPull;
		bool m_TransparentInvalid;

		// BASE overrides.
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// STAGE overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_PNG (const SOURCE_PNG&);
		SOURCE_PNG& operator= (const SOURCE_PNG&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEPNG_H

